//**************************************************************************
//**
//** $Id: i_linux.c,v 1.1.1.1 2000/04/11 17:38:04 theoddone33 Exp $
//**
//**************************************************************************


#include <stdlib.h>
#include <stdarg.h>
#include <sys/time.h>
#include "doomdef.h"
#include "r_local.h"
#include "p_local.h"    // for P_AproxDistance
#include "sounds.h"
#include "i_sound.h"
#include "soundst.h"

// Macros

#define stricmp strcasecmp
#define DEFAULT_ARCHIVEPATH     "o:\\sound\\archive\\"
#define PRIORITY_MAX_ADJUST 10
#define DIST_ADJUST (MAX_SND_DIST/PRIORITY_MAX_ADJUST)

#define KEY_LSHIFT      0xfe

#define KEY_INS         (0x80+0x52)
#define KEY_DEL         (0x80+0x53)
#define KEY_PGUP        (0x80+0x49)
#define KEY_PGDN        (0x80+0x51)
#define KEY_HOME        (0x80+0x47)
#define KEY_END         (0x80+0x4f)

extern void **lumpcache;

extern void I_StartupMouse();
extern void I_ShutdownGraphics();

int i_Vector;
externdata_t *i_ExternData;
boolean useexterndriver;

// Function Declarations
void I_StopSong(int handle);
void I_UnRegisterSong(int handle);
int I_RegisterSong (void *data);
void I_PlaySong(int handle, boolean looping);
int I_SoundIsPlaying (int handle);
void I_StopSound (int handle);
int I_GetSfxLumpNum (sfxinfo_t *sound);
int I_StartSound( int id, void* data, int vol, int sep, int pitch, int priority);
void I_PauseSong(int handle);
void I_ResumeSong(int handle);
void I_UpdateSoundParams(int handle, int vol, int sep, int pitch);
void I_StartupSound (void);
void I_SetChannels(int channels);
void I_SetMusicVolume(int volume);
void I_ShutdownSound (void);

/*
===============================================================================

		MUSIC & SFX API

===============================================================================
*/

//static channel_t channel[MAX_CHANNELS];

//static int rs; //the current registered song.
//int mus_song = -1;
//int mus_lumpnum;
//void *mus_sndptr;
//byte *soundCurve;

extern sfxinfo_t S_sfx[];
extern musicinfo_t S_music[];

static channel_t Channel[MAX_CHANNELS];
static int RegisteredSong; //the current registered song.
static int NextCleanup;
static boolean MusicPaused;
static int Mus_Song = -1;
static int Mus_LumpNum;
static void *Mus_SndPtr;
static byte *SoundCurve;

extern int snd_MusicDevice;
extern int snd_SfxDevice;
extern int snd_MaxVolume;
extern int snd_MusicVolume;
extern int snd_Channels;

extern int startepisode;
extern int startmap;

int AmbChan;

boolean S_StopSoundID(int sound_id, int priority);

//==========================================================================
//
// S_Start
//
//==========================================================================

void S_Start(void)
{
        int i;

        S_StartSong((gameepisode-1)*9 + gamemap-1, true);

        //stop all sounds
        for(i=0; i < snd_Channels; i++)
        {
                if(Channel[i].handle)
                {
                        S_StopSound(Channel[i].mo);
                }
        }
        memset(Channel, 0, 8*sizeof(channel_t));
}

//==========================================================================
//
// S_StartSong
//
//==========================================================================

void S_StartSong(int song, boolean loop)
{
	if(song == Mus_Song)
	{ // don't replay an old song
		return;
	}
	if(RegisteredSong)
	{
		I_StopSong(RegisteredSong);
		I_UnRegisterSong(RegisteredSong);
		Z_ChangeTag(lumpcache[Mus_LumpNum], PU_CACHE);
	}
        if(song < mus_e1m1 || song > NUMMUSIC)
        {
                return;
        }
	Mus_LumpNum = W_GetNumForName(S_music[song].name);
	Mus_SndPtr = W_CacheLumpNum(Mus_LumpNum, PU_MUSIC);
	RegisteredSong = I_RegisterSong(Mus_SndPtr);
	I_PlaySong(RegisteredSong, loop); // 'true' denotes endless looping.
	Mus_Song = song;
}

//==========================================================================
//
// S_StartSound
//
//==========================================================================

void S_StartSound(mobj_t *origin, int sound_id)
{
        int dist, vol;
        int i;
        int priority;
        int sep;
        int angle;
        int absx;
        int absy;

        static int sndcount = 0;
        int chan;

        if(sound_id==0 || snd_MaxVolume == 0)
                return;
        if(origin == NULL)
        {
        //      origin = players[consoleplayer].mo;
        }

// calculate the distance before other stuff so that we can throw out
// sounds that are beyond the hearing range.
	if (origin)
	{
		absx = abs(origin->x-players[consoleplayer].mo->x);
		absy = abs(origin->y-players[consoleplayer].mo->y);
	}
	else
		absx = absy = 0;
        dist = absx+absy-(absx > absy ? absy>>1 : absx>>1);
        dist >>= FRACBITS;
//  dist = P_AproxDistance(origin->x-viewx, origin->y-viewy)>>FRACBITS;

        if(dist >= MAX_SND_DIST)
        {
//      dist = MAX_SND_DIST - 1;
          return; //sound is beyond the hearing range...
        }
        if(dist < 0)
        {
                dist = 0;
        }
        priority = S_sfx[sound_id].priority;
        priority *= (10 - (dist/160));
        if(!S_StopSoundID(sound_id, priority))
        {
                return; // other sounds have greater priority
        }
        for(i=0; i<snd_Channels; i++)
        {
                if(!origin || origin->player)
                {
                        i = snd_Channels;
                        break; // let the player have more than one sound.
                }
                if(origin == Channel[i].mo)
                { // only allow other mobjs one sound
                        S_StopSound(Channel[i].mo);
                        break;
                }
        }
        if(i >= snd_Channels)
        {
                if(sound_id >= sfx_wind)
                {
                        if(AmbChan != -1 && S_sfx[sound_id].priority <=
                                S_sfx[Channel[AmbChan].sound_id].priority)
                        {
                                return; //ambient channel already in use
                        }
                        else
                        {
                                AmbChan = -1;
                        }
                }
                for(i=0; i<snd_Channels; i++)
                {
                        if(Channel[i].mo == NULL)
                        {
                                break;
                        }
                }
                if(i >= snd_Channels)
                {
                        //look for a lower priority sound to replace.
                        sndcount++;
                        if(sndcount >= snd_Channels)
                        {
                                sndcount = 0;
                        }
                        for(chan=0; chan < snd_Channels; chan++)
                        {
                                i = (sndcount+chan)%snd_Channels;
                                if(priority >= Channel[i].priority)
                                {
                                        chan = -1; //denote that sound should be replaced.
                                        break;
                                }
                        }
                        if(chan != -1)
                        {
                                return; //no free channels.
                        }
                        else //replace the lower priority sound.
                        {
                                if(Channel[i].handle)
                                {
                                        if(I_SoundIsPlaying(Channel[i].handle))
                                        {
                                                I_StopSound(Channel[i].handle);
                                        }
                                        if(S_sfx[Channel[i].sound_id].usefulness > 0)
                                        {
                                                S_sfx[Channel[i].sound_id].usefulness--;
                                        }

                                        if(AmbChan == i)
                                        {
                                                AmbChan = -1;
                                        }
                                }
                        }
                }
        }
        if(S_sfx[sound_id].lumpnum == 0)
        {
                S_sfx[sound_id].lumpnum = I_GetSfxLumpNum(&S_sfx[sound_id]);
        }
        if(S_sfx[sound_id].snd_ptr == NULL)
        {
                S_sfx[sound_id].snd_ptr = W_CacheLumpNum(S_sfx[sound_id].lumpnum,
                        PU_SOUND);
        }

        // calculate the volume based upon the distance from the sound origin.
//      vol = (snd_MaxVolume*16 + dist*(-snd_MaxVolume*16)/MAX_SND_DIST)>>9;
        vol = SoundCurve[dist];

        if(!origin || origin == players[consoleplayer].mo)
        {
                sep = 128;
        }
        else
        {
                angle = R_PointToAngle2(players[consoleplayer].mo->x,
//                        players[consoleplayer].mo->y, Channel[i].mo->x, Channel[i].mo->y);
			players[displayplayer].mo->y, origin->x, origin->y);

                angle = (angle-viewangle)>>24;
                sep = angle*2-128;
                if(sep < 64)
                        sep = -sep;
                if(sep > 192)
                        sep = 512-sep;
        }

        Channel[i].pitch = (byte)(127+(M_Random()&7)-(M_Random()&7));
        Channel[i].handle = I_StartSound(sound_id, S_sfx[sound_id].snd_ptr, vol, sep, Channel[i].pitch, 0);
        Channel[i].mo = origin;
        Channel[i].sound_id = sound_id;
        Channel[i].priority = priority;
        if(sound_id >= sfx_wind)
        {
                AmbChan = i;
        }
        if(S_sfx[sound_id].usefulness == -1)
        {
                S_sfx[sound_id].usefulness = 1;
        }
        else
        {
                S_sfx[sound_id].usefulness++;
        }
}

//==========================================================================
//
// S_StartSoundAtVolume
//
//==========================================================================
void S_StartSoundAtVolume(mobj_t *origin, int sound_id, int volume)
{
	int i;

	if(sound_id == 0 || snd_MaxVolume == 0)
		return;
	if(origin == NULL)
	{
		origin = players[displayplayer].mo;
	}

	if(volume == 0)
	{
		return;
	}
        volume = (volume*(snd_MaxVolume+1)*8)>>7;

// no priority checking, as ambient sounds would be the LOWEST.
        for(i=0; i<snd_Channels; i++)
        {
                if(Channel[i].mo == NULL)
                {
                        break;
                }
        }
        if(i >= snd_Channels)
        {
                return;
        }
        if(S_sfx[sound_id].lumpnum == 0)
        {
                S_sfx[sound_id].lumpnum = I_GetSfxLumpNum(&S_sfx[sound_id]);
        }
        if(S_sfx[sound_id].snd_ptr == NULL)
        {
                S_sfx[sound_id].snd_ptr = W_CacheLumpNum(S_sfx[sound_id].lumpnum,
                        PU_SOUND);
        }
        Channel[i].pitch = (byte)(127-(M_Random()&3)+(M_Random()&3));
        Channel[i].handle = I_StartSound(sound_id, S_sfx[sound_id].snd_ptr, volume, 128, Channel[i].pitch, 0);
        Channel[i].mo = origin;
        Channel[i].sound_id = sound_id;
        Channel[i].priority = 1; //super low priority.
        if(S_sfx[sound_id].usefulness == -1)
        {
                S_sfx[sound_id].usefulness = 1;
        }
        else
        {
                S_sfx[sound_id].usefulness++;
        }
}
//==========================================================================
//
// S_StopSoundID
//
//==========================================================================

boolean S_StopSoundID(int sound_id, int priority)
{
	int i;
	int lp; //least priority
	int found;

	if(S_sfx[sound_id].numchannels == -1)
	{
		return(true);
	}
	lp = -1; //denote the argument sound_id
	found = 0;
	for(i=0; i<snd_Channels; i++)
	{
		if(Channel[i].sound_id == sound_id && Channel[i].mo)
		{
			found++; //found one.  Now, should we replace it??
			if(priority >= Channel[i].priority)
			{ // if we're gonna kill one, then this'll be it
				lp = i;
				priority = Channel[i].priority;
			}
		}
	}
	if(found < S_sfx[sound_id].numchannels)
	{
		return(true);
	}
	else if(lp == -1)
	{
		return(false); // don't replace any sounds
	}
	if(Channel[lp].handle)
	{
		if(I_SoundIsPlaying(Channel[lp].handle))
		{
			I_StopSound(Channel[lp].handle);
		}
		if(S_sfx[Channel[lp].sound_id].usefulness > 0)
		{
			S_sfx[Channel[lp].sound_id].usefulness--;
		}
		Channel[lp].mo = NULL;
	}
	return(true);
}

//==========================================================================
//
// S_StopSound
//
//==========================================================================

void S_StopSound(mobj_t *origin)
{
	int i;

	for(i=0;i<snd_Channels;i++)
	{
		if(Channel[i].mo == origin)
		{
			I_StopSound(Channel[i].handle);
			if(S_sfx[Channel[i].sound_id].usefulness > 0)
			{
				S_sfx[Channel[i].sound_id].usefulness--;
			}
			Channel[i].handle = 0;
			Channel[i].mo = NULL;
                        if(AmbChan == i)
                        {
                                AmbChan = -1;
                        }
		}
	}
}

//==========================================================================
//
// S_SoundLink
//
//==========================================================================

void S_SoundLink(mobj_t *oldactor, mobj_t *newactor)
{
	int i;

	for(i=0;i<snd_Channels;i++)
	{
		if(Channel[i].mo == oldactor)
			Channel[i].mo = newactor;
	}
}

//==========================================================================
//
// S_PauseSound
//
//==========================================================================

void S_PauseSound(void)
{
	I_PauseSong(RegisteredSong);
}

//==========================================================================
//
// S_ResumeSound
//
//==========================================================================

void S_ResumeSound(void)
{
	I_ResumeSong(RegisteredSong);
}

//==========================================================================
//
// S_UpdateSounds
//
//==========================================================================

void S_UpdateSounds(mobj_t *listener)
{
	int i, dist, vol;
	int angle;
	int sep;
	int priority;
	int absx;
	int absy;

	listener = players[consoleplayer].mo;
	if(snd_MaxVolume == 0)
	{
		return;
	}

	if(NextCleanup < gametic)
	{
		for(i = 0; i < NUMSFX; i++)
		{
			if(S_sfx[i].usefulness == 0 && S_sfx[i].snd_ptr)
			{
				if(lumpcache[S_sfx[i].lumpnum])
				{
					if(((memblock_t *)((byte*) (lumpcache[S_sfx[i].lumpnum])-
						sizeof(memblock_t)))->id == 0x1d4a11)
					{ // taken directly from the Z_ChangeTag macro
						Z_ChangeTag2(lumpcache[S_sfx[i].lumpnum],
							PU_CACHE); 
					}
				}
				S_sfx[i].usefulness = -1;
				S_sfx[i].snd_ptr = NULL;
			}
		}
		// Note, heretic does this every second (gametic+35)
		NextCleanup = gametic+35*30; // every 30 seconds
	}
	for(i=0;i<snd_Channels;i++)
	{
		if(!Channel[i].handle || S_sfx[Channel[i].sound_id].usefulness == -1)
		{
			continue;
		}
		if(!I_SoundIsPlaying(Channel[i].handle))
		{
			if(S_sfx[Channel[i].sound_id].usefulness > 0)
			{
				S_sfx[Channel[i].sound_id].usefulness--;
			}
			Channel[i].handle = 0;
			Channel[i].mo = NULL;
			Channel[i].sound_id = 0;
                        if(AmbChan == i)
                        {
                                AmbChan = -1;
                        }
		}
		if(Channel[i].mo == NULL || Channel[i].sound_id == 0
			|| Channel[i].mo == listener)
		{
			continue;
		}
		else
		{
			absx = abs(Channel[i].mo->x-listener->x);
			absy = abs(Channel[i].mo->y-listener->y);
			dist = absx+absy-(absx > absy ? absy>>1 : absx>>1);
			dist >>= FRACBITS;

			if(dist >= MAX_SND_DIST)
			{
				S_StopSound(Channel[i].mo);
				continue;
			}
			if(dist < 0)
			{
				dist = 0;
			}
			vol = SoundCurve[dist];
			angle = R_PointToAngle2(listener->x, listener->y,
					Channel[i].mo->x, Channel[i].mo->y);
			angle = (angle-viewangle)>>24;
			sep = angle*2-128;
			if(sep < 64)
				sep = -sep;
			if(sep > 192)
				sep = 512-sep;
			I_UpdateSoundParams(Channel[i].handle, vol, sep, Channel[i].pitch);
			priority = S_sfx[Channel[i].sound_id].priority;
			priority *= PRIORITY_MAX_ADJUST-(dist/DIST_ADJUST);
			Channel[i].priority = priority;
		}
	}
}

//==========================================================================
//
// S_Init
//
//==========================================================================

void S_Init(void)
{
        SoundCurve = Z_Malloc(MAX_SND_DIST, PU_STATIC, NULL);
	I_StartupSound();
	if(snd_Channels > 8)
	{
		snd_Channels = 8;
	}
	I_SetChannels(snd_Channels);
	I_SetMusicVolume(snd_MusicVolume);
	S_SetMaxVolume(true);
}

//==========================================================================
//
// S_GetChannelInfo
//
//==========================================================================

void S_GetChannelInfo(SoundInfo_t *s)
{
	int i;
	ChanInfo_t *c;

	s->channelCount = snd_Channels;
	s->musicVolume = snd_MusicVolume;
	s->soundVolume = snd_MaxVolume;
	for(i = 0; i < snd_Channels; i++)
	{
		c = &s->chan[i];
		c->id = Channel[i].sound_id;
		c->priority = Channel[i].priority;
		c->name = S_sfx[c->id].name;
		c->mo = Channel[i].mo;
		c->distance = P_AproxDistance(c->mo->x-viewx, c->mo->y-viewy)
			>>FRACBITS;
	}
}

void S_SetMaxVolume(boolean fullprocess)
{
        int i;

        if(!fullprocess)
        {
                SoundCurve[0] = (*((byte *)W_CacheLumpName("SNDCURVE", PU_CACHE))*(snd_MaxVolume*8))>>7;
        }
        else
        {
                for(i = 0; i < MAX_SND_DIST; i++)
                {
                        SoundCurve[i] = (*((byte *)W_CacheLumpName("SNDCURVE", PU_CACHE)+i)*(snd_MaxVolume*8))>>7;
                }
        }
}

//==========================================================================
//
// S_SetMusicVolume
//
//==========================================================================

void S_SetMusicVolume(void)
{
	I_SetMusicVolume(snd_MusicVolume);
	if(snd_MusicVolume == 0)
	{
		I_PauseSong(RegisteredSong);
		MusicPaused = true;
	}
	else if(MusicPaused)
	{
		MusicPaused = false;
		I_ResumeSong(RegisteredSong);
	}
}

//==========================================================================
//
// S_ShutDown
//
//==========================================================================

void S_ShutDown(void)
{
	extern int tsm_ID;
	if(tsm_ID != -1)
	{
		I_StopSong(RegisteredSong);
		I_UnRegisterSong(RegisteredSong);
		I_ShutdownSound();
	}
}

//==================================================


int ticcount;

boolean novideo; // if true, stay in text mode for debugging


//==========================================================================


static long _startSec;

void I_StartupTimer(void)
{
   struct timeval tv;
    gettimeofday( &tv, 0 ); 
    _startSec = tv.tv_sec;
}

//--------------------------------------------------------------------------
//
// FUNC I_GetTime
//
// Returns time in 1/35th second tics.
//
//--------------------------------------------------------------------------

int I_GetTime (void)
{
    struct timeval tv;
    gettimeofday( &tv, 0 ); 

    //printf( "GT: %lx %lx\n", tv.tv_sec, tv.tv_usec );

//  ticcount = ((tv.tv_sec * 1000000) + tv.tv_usec) / 28571;
    ticcount = ((tv.tv_sec - _startSec) * 35) + (tv.tv_usec / 28571);
	return( ticcount );
}


/*
============================================================================

					JOYSTICK

============================================================================
*/

//==================================================
//
// joystick vars
//
//==================================================

extern int usejoystick;
boolean joystickpresent;
unsigned joystickx, joysticky;

// returns false if not connected
boolean I_ReadJoystick (void)
{
    return false;
}


int     joyxl, joyxh, joyyl, joyyh;

boolean WaitJoyButton (void)
{
#if 0
	int             oldbuttons, buttons;

	oldbuttons = 0;
	do
	{
		I_WaitVBL (1);
		buttons =  ((inp(0x201) >> 4)&1)^1;
		if (buttons != oldbuttons)
		{
			oldbuttons = buttons;
			continue;
		}

		if ( (lastpress& 0x7f) == 1 )
		{
			joystickpresent = false;
			return false;
		}
	} while ( !buttons);

	do
	{
		I_WaitVBL (1);
		buttons =  ((inp(0x201) >> 4)&1)^1;
		if (buttons != oldbuttons)
		{
			oldbuttons = buttons;
			continue;
		}

		if ( (lastpress& 0x7f) == 1 )
		{
			joystickpresent = false;
			return false;
		}
	} while ( buttons);

#endif
	return true;
}



/*
===============
=
= I_StartupJoystick
=
===============
*/

int             basejoyx, basejoyy;

void I_StartupJoystick (void)
{
	int     centerx, centery;

	joystickpresent = 0;
	if ( M_CheckParm ("-nojoy") || !usejoystick )
		return;

	if (!I_ReadJoystick ())
	{
		joystickpresent = false;
		tprintf ("joystick not found\n ",1);
		return;
	}
	tprintf ("joystick found\n",1);
	joystickpresent = true;

	tprintf ("CENTER the joystick and press button 1:",1);
	if (!WaitJoyButton ())
		return;
	I_ReadJoystick ();
	centerx = joystickx;
	centery = joysticky;

	tprintf("\nPush the joystick to the UPPER LEFT corner and press button 1:",1);
	if (!WaitJoyButton ())
		return;
	I_ReadJoystick ();
	joyxl = (centerx + joystickx)/2;
	joyyl = (centerx + joysticky)/2;

	tprintf("\nPush the joystick to the LOWER RIGHT corner and press button 1:",1);
	if (!WaitJoyButton ())
		return;
	I_ReadJoystick ();
	joyxh = (centerx + joystickx)/2;
	joyyh = (centery + joysticky)/2;
	tprintf ("\n", 1);
}

/*
===============
=
= I_JoystickEvents
=
===============
*/

void I_JoystickEvents (void)
{
	event_t ev;

//
// joystick events
//
	if (!joystickpresent)
		return;

	I_ReadJoystick();
	ev.type = ev_joystick;
	ev.data1 =  0;  //((inp(0x201) >> 4)&15)^15;

	if (joystickx < joyxl)
		ev.data2 = -1;
	else if (joystickx > joyxh)
		ev.data2 = 1;
	else
		ev.data2 = 0;
	if (joysticky < joyyl)
		ev.data3 = -1;
	else if (joysticky > joyyh)
		ev.data3 = 1;
	else
		ev.data3 = 0;

	D_PostEvent (&ev);
}


/*
   ===============
   =
   = I_StartFrame
   =
   ===============
   */
 
void I_StartFrame (void)
{
    I_JoystickEvents();
}


/*
============================================================================

					DPMI STUFF

============================================================================
*/

void I_DivException (void);
int I_SetDivException (void);



/*
============================================================================

					TIMER INTERRUPT

============================================================================
*/


/*
===============
=
= I_Init
=
= hook interrupts and set graphics mode
=
===============
*/

void I_Init (void)
{
	void I_StartupTimer(void);

	novideo = M_CheckParm("novideo");
	I_StartupMouse();
//	tprintf("I_StartupJoystick ",1);
//	I_StartupJoystick();
	tprintf("S_Init... ",1);
	S_Init();
	S_Start();

    I_StartupTimer();
}


/*
===============
=
= I_Shutdown
=
= return to default system state
=
===============
*/

void I_Shutdown (void)
{
	I_ShutdownGraphics ();
	S_ShutDown ();
}


/*
================
=
= I_Error
=
================
*/

void I_Error (char *error, ...)
{
	va_list argptr;

	D_QuitNetGame ();
	I_Shutdown ();
	va_start (argptr,error);
	vprintf (error, argptr);
	va_end (argptr);
	printf ("\n");
	exit (1);
}

//--------------------------------------------------------------------------
//
// I_Quit
//
// Shuts down net game, saves defaults, prints the exit text message,
// goes to text mode, and exits.
//
//--------------------------------------------------------------------------

void put_dos2ansi (byte attrib)
{
	byte fore,back,blink=0,intens=0;
	int table[] = {30,34,32,36,31,35,33,37};
	
	fore = attrib&15;	// bits 0-3
	back = attrib&112; // bits 4-6
       	blink = attrib&128; // bit 7
	
	// Fix background, blink is either on or off.
	back = back>>4;

	// Fix foreground
	if (fore > 7) {
		intens = 1;
		fore-=8;
	}

	// Convert fore/back
	fore = table[fore];
	back = table[back] + 10;

	// 'Render'
	if (blink)
		printf ("\033[%d;5;%dm\033[%dm", intens, fore, back);
	else
		printf ("\033[%d;25;%dm\033[%dm", intens, fore, back);
}

void I_Quit(void)
{
	int i;
	byte *scr;
	
	D_QuitNetGame();
	M_SaveDefaults();
	I_Shutdown();

	scr = (byte *)W_CacheLumpName("ENDTEXT", PU_CACHE);
/*
	memcpy((void *)0xb8000, scr, 80*25*2);
	regs.w.ax = 0x0200;
	regs.h.bh = 0;
	regs.h.dl = 0;
	regs.h.dh = 23;
	int386(0x10, (const union REGS *)&regs, &regs); // Set text pos
	_settextposition(24, 1);
*/
	for (i=0; i< 80*25*2; i+=2) {
		put_dos2ansi(scr[i+1]);
		putchar (scr[i]);
	}
	// Cleanup after all that ANSI stuff
	printf ("\033[m");
	exit(0);
}

/*
===============
=
= I_ZoneBase
=
===============
*/

byte *I_ZoneBase (int *size)
{
	byte *ptr;
	int heap = 0x800000;

	ptr = malloc ( heap );

	if ( ! ptr )
		I_Error ("  Insufficient DPMI memory!");

	*size = heap;
	return ptr;
}

/*
=============
=
= I_AllocLow
=
=============
*/

byte *I_AllocLow (int length)
{
	return malloc( length );
}

/*
============================================================================

						NETWORKING

============================================================================
*/

/* // FUCKED LINES
typedef struct
{
	char    priv[508];
} doomdata_t;
*/ // FUCKED LINES

#define DOOMCOM_ID              0x12345678l

/* // FUCKED LINES
typedef struct
{
	long    id;
	short   intnum;                 // DOOM executes an int to execute commands

// communication between DOOM and the driver
	short   command;                // CMD_SEND or CMD_GET
	short   remotenode;             // dest for send, set by get (-1 = no packet)
	short   datalength;             // bytes in doomdata to be sent

// info common to all nodes
	short   numnodes;               // console is allways node 0
	short   ticdup;                 // 1 = no duplication, 2-5 = dup for slow nets
	short   extratics;              // 1 = send a backup tic in every packet
	short   deathmatch;             // 1 = deathmatch
	short   savegame;               // -1 = new game, 0-5 = load savegame
	short   episode;                // 1-3
	short   map;                    // 1-9
	short   skill;                  // 1-5

// info specific to this node
	short   consoleplayer;
	short   numplayers;
	short   angleoffset;    // 1 = left, 0 = center, -1 = right
	short   drone;                  // 1 = drone

// packet data to be sent
	doomdata_t      data;
} doomcom_t;
*/ // FUCKED LINES

extern  doomcom_t               *doomcom;

/*
====================
=
= I_InitNetwork
=
====================
*/

void I_InitNetwork (void)
{
	int             i;

	i = M_CheckParm ("-net");
	if (!i)
	{
	//
	// single player game
	//
		doomcom = malloc (sizeof (*doomcom) );
		memset (doomcom, 0, sizeof(*doomcom) );
		netgame = false;
		doomcom->id = DOOMCOM_ID;
		doomcom->numplayers = doomcom->numnodes = 1;
		doomcom->deathmatch = false;
		doomcom->consoleplayer = 0;
		doomcom->ticdup = 1;
		doomcom->extratics = 0;
		return;
	}
	netgame = true;
	doomcom = (doomcom_t *)atoi(myargv[i+1]);
//DEBUG
doomcom->skill = startskill;
doomcom->episode = startepisode;
doomcom->map = startmap;
doomcom->deathmatch = deathmatch;
}

void I_NetCmd (void)
{
	if (!netgame)
		I_Error ("I_NetCmd when not in netgame");
}

//=========================================================================
//
// I_CheckExternDriver
//
//		Checks to see if a vector, and an address for an external driver
//			have been passed.
//=========================================================================

void I_CheckExternDriver(void)
{
	int i;

	if(!(i = M_CheckParm("-externdriver")))
	{
		return;
	}
	i_ExternData = (externdata_t *)atoi(myargv[i+1]);
	i_Vector = i_ExternData->vector;

	useexterndriver = true;
}


int main( int argc, char** argv )
{
	myargc = argc;
	myargv = argv;
	D_DoomMain();
	return 0;
}

fixed_t FixedMul (fixed_t a, fixed_t b)
{
    return ((long long) a * (long long) b) >> 16;
}

fixed_t FixedDiv2 (fixed_t a, fixed_t b)
{
    double c;

    c = ((double)a) / ((double)b) * FRACUNIT;

    if (c >= 2147483648.0 || c < -2147483648.0)
        I_Error("FixedDiv: divide by zero");
    return (fixed_t) c;
}

//EOF
